@@ -1,6 +1,6 @@
-/* $Id: display_plugin.c,v 1.9 2004/10/24 15:33:09 benny Exp $ */
+/* $Id: display_plugin.c 21493 2006-05-04 21:26:41Z benny $ */
 /*-
- * Copyright (c) 2003-2004 Benedikt Meurer <benny@xfce.org>
+ * Copyright (c) 2003-2006 Benedikt Meurer <benny@xfce.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -49,10 +49,14 @@
 #define GAMMA_MAX	10.0
 
 /* */
-#define	RCDIR		  "mcs_settings"
+#define	RCDIR		"mcs_settings"
 #define	OLDRCDIR	"settings"
 #define	CHANNEL		"display"
 #define	RCFILE		"display.xml"
+#define DEFAULT_RES     -1
+
+/* */
+#define CONFIRM_TIMEOUT 15
 
 /* */
 enum
@@ -64,22 +68,44 @@ enum
 };
 
 /* settings */
-static int sizeIndex = 0;
-static int rateIndex = 0;
+static int sizeIndex = DEFAULT_RES;
+static int rateIndex = DEFAULT_RES;
 static int redValue = 100;
 static int greenValue = 100;
 static int blueValue = 100;
 static gboolean syncGamma = TRUE;
+static int default_size = 0;
+static int default_rate = 0;
 
 /* supported extensions */
 static gboolean haveXrandr = FALSE;
 static gboolean haveXxf86vm = FALSE;
 
 /* */
-static GtkWidget *dialog = NULL;
-static GtkWidget *rscale = NULL;
-static GtkWidget *gscale = NULL;
-static GtkWidget *bscale = NULL;
+typedef struct _Itf Itf;
+struct _Itf {
+    McsPlugin *plugin;
+
+    GtkWidget *dialog;
+    
+    GtkWidget *treeview;
+    GtkListStore *store;
+
+    GtkWidget *rscale;
+    GtkWidget *gscale;
+    GtkWidget *bscale;
+
+    GtkWidget *synctoggle;
+
+    int original_size;
+    int original_rate;
+
+    int original_rgamma;
+    int original_ggamma;
+    int original_bgamma;
+
+    gboolean original_sync;
+};
 
 /* static prototypes */
 static void run_dialog (McsPlugin *);
@@ -123,6 +149,60 @@ change_size_and_rate (XRRScreenConfiguration * sc, int size, int rate)
         return;
       }
 }
+
+static gboolean
+confirm_timeout_cb (gpointer user_data)
+{
+    gint timeout;
+    GtkWidget *msgdlg = user_data;
+
+    timeout = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (msgdlg), "timeout"));
+
+    if (!--timeout)
+    {
+        gtk_dialog_response (GTK_DIALOG (msgdlg), GTK_RESPONSE_NO);
+        return FALSE;
+    }
+
+    gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (msgdlg),
+                                              _("Old settings will be restored in %d seconds"),
+                                              timeout);
+    g_object_set_data (G_OBJECT (msgdlg), "timeout", GINT_TO_POINTER (timeout));
+
+    return TRUE;
+}
+
+static gboolean
+confirm_display_mode (GtkWidget * parent)
+{
+    GtkWidget *msgdlg;
+    gint response;
+    guint timeout_id;
+
+    msgdlg = gtk_message_dialog_new (GTK_WINDOW (parent),
+                                     GTK_DIALOG_MODAL |
+                                     GTK_DIALOG_DESTROY_WITH_PARENT,
+                                     GTK_MESSAGE_QUESTION,
+                                     GTK_BUTTONS_YES_NO,
+                                     _("Display settings have been changed.\n"
+                                       "Would you like to keep these settings?"));
+
+    gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (msgdlg),
+                                              _("Previous settings will be restored in %d seconds"),
+                                              CONFIRM_TIMEOUT);
+
+    gtk_window_set_position (GTK_WINDOW (msgdlg), GTK_WIN_POS_CENTER_ALWAYS);
+
+    g_object_set_data (G_OBJECT (msgdlg), "timeout", GINT_TO_POINTER (CONFIRM_TIMEOUT));
+    timeout_id = g_timeout_add (1000, confirm_timeout_cb, msgdlg);
+
+    response = gtk_dialog_run (GTK_DIALOG (msgdlg));
+
+    g_source_remove (timeout_id);
+    gtk_widget_destroy (msgdlg);
+    
+    return response == GTK_RESPONSE_YES;
+}
 #endif
 
 #ifdef USE_XF86VM
@@ -245,15 +325,34 @@ mcs_plugin_init (McsPlugin * plugin)
 #ifdef USE_XRANDR
     if (XRRQueryVersion (GDK_DISPLAY (), &major, &minor))
     {
-        /* restore resolution and rate */
         if ((sc = XRRGetScreenInfo (GDK_DISPLAY (), GDK_ROOT_WINDOW ())))
         {
-            change_size_and_rate (sc, sizeIndex, rateIndex);
+            Rotation current_rotation;
+            
+            gdk_error_trap_push ();
+            
+            /* Save default values */
+            default_size = XRRConfigCurrentConfiguration (sc, &current_rotation);
+            default_rate = XRRConfigCurrentRate (sc);
+
+            if (!gdk_error_trap_pop ())
+            {
+                if (sizeIndex >= 0 && rateIndex >= 0)
+                {
+                    /* restore resolution and rate */
+                    change_size_and_rate (sc, sizeIndex, rateIndex);
+                }
+                else {
+                    sizeIndex = default_size;
+                    rateIndex = default_rate;
+                }
+
+                /* remember that we can use the Xrandr extension */
+                haveXrandr = TRUE;
+            }
+            
             XRRFreeScreenConfigInfo (sc);
         }
-
-        /* remember that we can use the Xrandr extension */
-        haveXrandr = TRUE;
     }
 #endif
 
@@ -276,11 +375,17 @@ mcs_plugin_init (McsPlugin * plugin)
 #endif
 
     plugin->plugin_name = g_strdup ("display");
-    plugin->caption = g_strdup (_("Display"));
+    /* the button label in the xfce-mcs-manager dialog */
+    plugin->caption = g_strdup (Q_ ("Button Label|Display"));
     plugin->run_dialog = run_dialog;
     plugin->icon = xfce_themed_icon_load ("xfce4-display", 48);
+    if (G_LIKELY (plugin->icon != NULL))
+      g_object_set_data_full (G_OBJECT (plugin->icon), "mcs-plugin-icon-name", g_strdup ("xfce4-display"), g_free);
 
+#if 0    
+    /* I fail to see why we need to save the options here, during startup... */
     save_settings (plugin);
+#endif
 
     return (MCS_PLUGIN_INIT_OK);
 }
@@ -290,15 +395,17 @@ mcs_plugin_init (McsPlugin * plugin)
 static gboolean
 save_settings (McsPlugin * plugin)
 {
-    gboolean result;
+    gboolean result = FALSE;
     gchar *rcfile, *path;
 
     path = g_build_filename ("xfce4", RCDIR, RCFILE, NULL);
     rcfile = xfce_resource_save_location (XFCE_RESOURCE_CONFIG, path, TRUE);
-    result = mcs_manager_save_channel_to_file(plugin->manager, CHANNEL, 
-                                              rcfile);
+    if (G_LIKELY (rcfile != NULL))
+    {
+        result = mcs_manager_save_channel_to_file (plugin->manager, CHANNEL, rcfile);
+        g_free(rcfile);
+    }
     g_free(path);
-    g_free(rcfile);
 
     return (result);
 }
@@ -307,27 +414,77 @@ save_settings (McsPlugin * plugin)
 /*
  */
 static void
-changedCB (GtkTreeSelection * selection, McsPlugin * plugin)
+changedCB (GtkTreeSelection * selection, Itf * itf)
 {
     XRRScreenConfiguration *sc;
     GtkTreeModel *store;
     GtkTreeIter iter;
+    int newsize, newrate;
+    gboolean is_default;
+    /* If new resolution is rejected, we get called
+     * again while restoring the old resolution */
+    static guint recursion_count = 0;
 
     if (!gtk_tree_selection_get_selected (selection, &store, &iter))
         return;
 
-    gtk_tree_model_get (store, &iter, RATE_COLUMN, &rateIndex, SIZE_COLUMN, &sizeIndex, -1);
+    recursion_count++;
+
+    gtk_tree_model_get (store, &iter,
+                        SIZE_COLUMN, &newsize,
+                        RATE_COLUMN, &newrate,
+                        -1);
 
     /* apply settings */
     sc = XRRGetScreenInfo (GDK_DISPLAY (), GDK_ROOT_WINDOW ());
-    change_size_and_rate (sc, sizeIndex, rateIndex);
+    change_size_and_rate (sc, newsize, newrate);
     XRRFreeScreenConfigInfo (sc);
 
-    /* and remember the settings */
-    mcs_manager_set_int (plugin->manager, "XDisplay/rate", CHANNEL, rateIndex);
-    mcs_manager_set_int (plugin->manager, "XDisplay/size", CHANNEL, sizeIndex);
-    mcs_manager_notify (plugin->manager, CHANNEL);
-    save_settings (plugin);
+    if (recursion_count > 1)
+    {
+        recursion_count--;
+        return;
+    }
+
+    is_default = ( newsize == default_size && newrate == default_rate );
+
+    if (!is_default &&
+        !(newsize == itf->original_size && newrate == itf->original_rate))
+    {
+        if (!confirm_display_mode (itf->dialog))
+        {
+            /* Restore previous */
+            if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (store), &iter))
+            {
+                do {
+                    gtk_tree_model_get (store, &iter,
+                                        RATE_COLUMN, &newrate,
+                                        SIZE_COLUMN, &newsize,
+                                        -1);
+
+                    if (newrate == rateIndex && newsize == sizeIndex)
+                    {
+                        gtk_tree_selection_select_iter (selection, &iter);
+                        break;
+                    }
+                } while (gtk_tree_model_iter_next (GTK_TREE_MODEL (store), &iter));
+            }
+        }
+    }
+
+    if (newsize != sizeIndex || newrate != rateIndex) {
+        sizeIndex = newsize;
+        rateIndex = newrate;
+        
+        mcs_manager_set_int (itf->plugin->manager, "XDisplay/rate", CHANNEL,
+                             is_default ? DEFAULT_RES : newrate);
+        mcs_manager_set_int (itf->plugin->manager, "XDisplay/size", CHANNEL,
+                             is_default ? DEFAULT_RES : newsize);
+        mcs_manager_notify (itf->plugin->manager, CHANNEL);
+        save_settings (itf->plugin);
+    }
+
+    recursion_count--;
 }
 #endif
 
@@ -335,7 +492,7 @@ changedCB (GtkTreeSelection * selection, McsPlugin * plugin)
 /*
  */
 static void
-redChangedCB (GtkRange * range, McsPlugin * plugin)
+redChangedCB (GtkRange * range, Itf * itf)
 {
     double value;
 
@@ -347,25 +504,25 @@ redChangedCB (GtkRange * range, McsPlugin * plugin)
     if (syncGamma)
     {
         if (greenValue * 100.0 != value)
-            gtk_range_set_value (GTK_RANGE (gscale), value);
+            gtk_range_set_value (GTK_RANGE (itf->gscale), value);
 
         if (blueValue * 100.0 != value)
-            gtk_range_set_value (GTK_RANGE (bscale), value);
+            gtk_range_set_value (GTK_RANGE (itf->bscale), value);
     }
 
     /* apply settings */
     change_gamma (redValue, greenValue, blueValue);
 
     /* and remember the settings */
-    mcs_manager_set_int (plugin->manager, "XDisplay/rgamma", CHANNEL, redValue);
-    mcs_manager_notify (plugin->manager, CHANNEL);
-    save_settings (plugin);
+    mcs_manager_set_int (itf->plugin->manager, "XDisplay/rgamma", CHANNEL, redValue);
+    mcs_manager_notify (itf->plugin->manager, CHANNEL);
+    save_settings (itf->plugin);
 }
 
 /*
  */
 static void
-greenChangedCB (GtkRange * range, McsPlugin * plugin)
+greenChangedCB (GtkRange * range, Itf * itf)
 {
     double value;
 
@@ -377,25 +534,25 @@ greenChangedCB (GtkRange * range, McsPlugin * plugin)
     if (syncGamma)
     {
         if (redValue * 100.0 != value)
-            gtk_range_set_value (GTK_RANGE (rscale), value);
+            gtk_range_set_value (GTK_RANGE (itf->rscale), value);
 
         if (blueValue * 100.0 != value)
-            gtk_range_set_value (GTK_RANGE (bscale), value);
+            gtk_range_set_value (GTK_RANGE (itf->bscale), value);
     }
 
     /* apply settings */
     change_gamma (redValue, greenValue, blueValue);
 
     /* and remember the settings */
-    mcs_manager_set_int (plugin->manager, "XDisplay/ggamma", CHANNEL, greenValue);
-    mcs_manager_notify (plugin->manager, CHANNEL);
-    save_settings (plugin);
+    mcs_manager_set_int (itf->plugin->manager, "XDisplay/ggamma", CHANNEL, greenValue);
+    mcs_manager_notify (itf->plugin->manager, CHANNEL);
+    save_settings (itf->plugin);
 }
 
 /*
  */
 static void
-blueChangedCB (GtkRange * range, McsPlugin * plugin)
+blueChangedCB (GtkRange * range, Itf * itf)
 {
     double value;
 
@@ -407,25 +564,25 @@ blueChangedCB (GtkRange * range, McsPlugin * plugin)
     if (syncGamma)
     {
         if (redValue * 100.0 != value)
-            gtk_range_set_value (GTK_RANGE (rscale), value);
+            gtk_range_set_value (GTK_RANGE (itf->rscale), value);
 
         if (greenValue * 100.0 != value)
-            gtk_range_set_value (GTK_RANGE (gscale), value);
+            gtk_range_set_value (GTK_RANGE (itf->gscale), value);
     }
 
     /* apply settings */
     change_gamma (redValue, greenValue, blueValue);
 
     /* and remember the settings */
-    mcs_manager_set_int (plugin->manager, "XDisplay/bgamma", CHANNEL, blueValue);
-    mcs_manager_notify (plugin->manager, CHANNEL);
-    save_settings (plugin);
+    mcs_manager_set_int (itf->plugin->manager, "XDisplay/bgamma", CHANNEL, blueValue);
+    mcs_manager_notify (itf->plugin->manager, CHANNEL);
+    save_settings (itf->plugin);
 }
 
 /*
  */
 static void
-syncGammaChangedCB (GtkToggleButton * button, McsPlugin * plugin)
+syncGammaChangedCB (GtkToggleButton * button, Itf * itf)
 {
     gint value;
 
@@ -434,14 +591,14 @@ syncGammaChangedCB (GtkToggleButton * button, McsPlugin * plugin)
     if (syncGamma)
     {
         value = (redValue + greenValue + blueValue) / 3;
-        gtk_range_set_value (GTK_RANGE (rscale), value / 100.0);
-        gtk_range_set_value (GTK_RANGE (gscale), value / 100.0);
-        gtk_range_set_value (GTK_RANGE (bscale), value / 100.0);
+        gtk_range_set_value (GTK_RANGE (itf->rscale), value / 100.0);
+        gtk_range_set_value (GTK_RANGE (itf->gscale), value / 100.0);
+        gtk_range_set_value (GTK_RANGE (itf->bscale), value / 100.0);
     }
 
-    mcs_manager_set_int (plugin->manager, "XDisplay/syncGamma", CHANNEL, syncGamma);
-    mcs_manager_notify (plugin->manager, CHANNEL);
-    save_settings (plugin);
+    mcs_manager_set_int (itf->plugin->manager, "XDisplay/syncGamma", CHANNEL, syncGamma);
+    mcs_manager_notify (itf->plugin->manager, CHANNEL);
+    save_settings (itf->plugin);
 }
 
 #endif /* !USE_XF86VM */
@@ -449,20 +606,68 @@ syncGammaChangedCB (GtkToggleButton * button, McsPlugin * plugin)
 /*
  */
 static void
-responseCB (GtkWidget * widget, gint response, McsPlugin * plugin)
+responseCB (GtkWidget * widget, gint response, Itf * itf)
 {
     switch (response)
     {
         case GTK_RESPONSE_CANCEL:
-            /* revert settings */
-            gtk_range_set_value (GTK_RANGE (rscale), 1.0);
-            gtk_range_set_value (GTK_RANGE (gscale), 1.0);
-            gtk_range_set_value (GTK_RANGE (bscale), 1.0);
+            {
+                /* revert settings */
+                if (haveXrandr)
+                {
+#ifdef USE_XRANDR
+                    GtkTreeIter iter;
+
+                    if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (itf->store), &iter))
+                    {
+                        gint size, rate;
+
+                        do {
+                            gtk_tree_model_get (GTK_TREE_MODEL (itf->store), &iter,
+                                                SIZE_COLUMN, &size,
+                                                RATE_COLUMN, &rate,
+                                                -1 );
+
+                            if (size == itf->original_size && rate == itf->original_rate)
+                            {
+                                GtkTreePath *path;
+                                
+                                path = gtk_tree_model_get_path (GTK_TREE_MODEL (itf->store), &iter);
+                                gtk_tree_view_set_cursor (GTK_TREE_VIEW (itf->treeview), path,
+                                                          0, FALSE);
+                                gtk_tree_path_free (path);
+                            }
+                        } while (gtk_tree_model_iter_next (GTK_TREE_MODEL (itf->store), &iter));
+                    }
+#endif /* !USE_XRANDR */
+                }
+
+                if (haveXxf86vm)
+                {
+#ifdef USE_XF86VM
+                    gint rgamma, ggamma, bgamma;
+
+                    rgamma = itf->original_rgamma;
+                    ggamma = itf->original_ggamma;
+                    bgamma = itf->original_bgamma;
+
+                    if (itf->original_sync)
+                    {
+                        rgamma = ggamma = bgamma = (rgamma + ggamma + bgamma) / 3;
+                    }
+
+                    gtk_range_set_value (GTK_RANGE (itf->rscale), rgamma / 100);
+                    gtk_range_set_value (GTK_RANGE (itf->gscale), ggamma / 100);
+                    gtk_range_set_value (GTK_RANGE (itf->bscale), bgamma / 100);
+                    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (itf->synctoggle),
+                                                  itf->original_sync);
+#endif /* !USE_XF86VM */
+                }
+            }
             break;
 
         default:
-            gtk_widget_destroy (dialog);
-            dialog = NULL;
+            gtk_widget_destroy (itf->dialog);
     }
 }
 
@@ -475,7 +680,6 @@ run_dialog (McsPlugin * plugin)
     GtkListStore *store;
     GtkWidget *revert;
     GtkWidget *close;
-    GtkWidget *header;
     GtkWidget *hbox;
     GtkWidget *align;
     GtkWidget *frame;
@@ -487,16 +691,28 @@ run_dialog (McsPlugin * plugin)
     GtkWidget *glabel;
     GtkWidget *blabel;
     GtkWidget *checkbox;
+    GtkWidget *dialog;
+    GtkWidget *rscale;
+    GtkWidget *gscale;
+    GtkWidget *bscale;
+    static Itf *itf = NULL;
 
-    if (dialog != NULL)
+    if (itf != NULL)
     {
-        gtk_window_present (GTK_WINDOW (dialog));
+        gtk_window_present (GTK_WINDOW (itf->dialog));
         return;
     }
 
     xfce_textdomain (GETTEXT_PACKAGE, LOCALEDIR, "UTF-8");
 
-    dialog = gtk_dialog_new_with_buttons (_("Display Preferences"), NULL, GTK_DIALOG_NO_SEPARATOR, NULL);
+    itf = g_new( Itf, 1 );
+
+    itf->plugin = plugin;
+
+    itf->dialog = dialog = xfce_titled_dialog_new_with_buttons (_("Display Preferences"), NULL, GTK_DIALOG_NO_SEPARATOR, NULL);
+    gtk_window_set_icon_name (GTK_WINDOW (dialog), "xfce4-display");
+    g_signal_connect_swapped (G_OBJECT (dialog), "destroy", G_CALLBACK (g_free), itf);
+    g_signal_connect (G_OBJECT (dialog), "destroy", G_CALLBACK (gtk_widget_destroyed), &itf);
 
     /* action widgets */
     gtk_button_box_set_layout (GTK_BUTTON_BOX (GTK_DIALOG (dialog)->action_area), GTK_BUTTONBOX_EDGE);
@@ -510,14 +726,10 @@ run_dialog (McsPlugin * plugin)
     gtk_widget_grab_default (close);
     gtk_widget_grab_focus (close);
 
-    gtk_window_set_icon (GTK_WINDOW (dialog), plugin->icon);
     gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER);
     gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
 
-    g_signal_connect_swapped (dialog, "response", G_CALLBACK (responseCB), plugin);
-
-    header = xfce_create_header (plugin->icon, _("Display Preferences"));
-    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), header, FALSE, TRUE, 0);
+    g_signal_connect (dialog, "response", G_CALLBACK (responseCB), itf);
 
     /* */
     hbox = gtk_hbox_new (FALSE, BORDER);
@@ -544,10 +756,13 @@ run_dialog (McsPlugin * plugin)
     gtk_box_pack_start (GTK_BOX (vbox), scrollwin, TRUE, TRUE, 0);
 
     /* create store */
-    store = gtk_list_store_new (N_COLUMNS, G_TYPE_STRING, G_TYPE_INT, G_TYPE_INT);
+    itf->store = store = gtk_list_store_new (N_COLUMNS,
+                                             G_TYPE_STRING,
+                                             G_TYPE_INT, G_TYPE_INT,
+                                             G_TYPE_BOOLEAN);
 
     /* create tree view */
-    treeview = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
+    itf->treeview = treeview = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
     gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (treeview), TRUE);
     gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (treeview), FALSE);
     gtk_widget_set_size_request (treeview, -1, 200);
@@ -568,6 +783,8 @@ run_dialog (McsPlugin * plugin)
         int nsizes, i, j;
         int current_rate;
         int current_size;
+        GtkTreePath *path;
+        GtkTreeIter iter;
 
         sc = XRRGetScreenInfo (GDK_DISPLAY (), GDK_ROOT_WINDOW ());
 
@@ -578,10 +795,19 @@ run_dialog (McsPlugin * plugin)
         current_rate = XRRConfigCurrentRate (sc);
         current_size = XRRConfigCurrentConfiguration (sc, &current_rotation);
 
+        /* Add default entry */
+        gtk_list_store_append (store, &iter);
+        gtk_list_store_set (store, &iter, NAME_COLUMN, "Default",
+                            SIZE_COLUMN, default_size,
+                            RATE_COLUMN, default_rate,
+                            DEFAULT_RES);
+        path = gtk_tree_model_get_path (GTK_TREE_MODEL (store), &iter);
+        gtk_tree_view_set_cursor (GTK_TREE_VIEW (treeview), path, 0, FALSE);
+        gtk_tree_path_free (path);
+
         /* fill store */
         for (i = 0; i < nsizes; i++)
         {
-            GtkTreeIter iter;
             gchar *buffer;
             short *rates;
             int nrates;
@@ -593,16 +819,20 @@ run_dialog (McsPlugin * plugin)
             {
                 buffer = g_strdup_printf (_("%dx%d@%d"), sizes[i].width, sizes[i].height, (int) rates[j]);
 
-                gtk_list_store_append (store, &iter);
-                gtk_list_store_set (store, &iter, NAME_COLUMN, buffer, SIZE_COLUMN, (int) i, RATE_COLUMN, (int) rates[j], -1);
-
-                if (i == current_size && rates[j] == current_rate)
+                if (i != default_size || rates[j] != default_rate)
                 {
-                    GtkTreePath *path;
+                    gtk_list_store_append (store, &iter);
+                    gtk_list_store_set (store, &iter,
+                                        NAME_COLUMN, buffer,
+                                        SIZE_COLUMN, (int) i, RATE_COLUMN, (int) rates[j],
+                                        -1);
 
-                    path = gtk_tree_model_get_path (GTK_TREE_MODEL (store), &iter);
-                    gtk_tree_view_set_cursor (GTK_TREE_VIEW (treeview), path, 0, FALSE);
-                    gtk_tree_path_free (path);
+                    if (i == current_size && rates[j] == current_rate)
+                    {
+                        path = gtk_tree_model_get_path (GTK_TREE_MODEL (store), &iter);
+                        gtk_tree_view_set_cursor (GTK_TREE_VIEW (treeview), path, 0, FALSE);
+                        gtk_tree_path_free (path);
+                    }
                 }
 
                 g_free (buffer);
@@ -613,7 +843,11 @@ run_dialog (McsPlugin * plugin)
         XRRFreeScreenConfigInfo (sc);
 
         /* connect the changed callback */
-        g_signal_connect (G_OBJECT (selection), "changed", G_CALLBACK (changedCB), plugin);
+        g_signal_connect (G_OBJECT (selection), "changed", G_CALLBACK (changedCB), itf);
+
+        /* Store values to revert to */
+        itf->original_size = current_size;
+        itf->original_rate = current_rate;
 #endif /* !USE_XRANDR */
     }
     else
@@ -643,7 +877,7 @@ run_dialog (McsPlugin * plugin)
     /* Red */
     rlabel = gtk_label_new (_("Red"));
     gtk_table_attach (GTK_TABLE (table), rlabel, 0, 1, 0, 1, GTK_FILL, GTK_FILL, BORDER, BORDER);
-    rscale = gtk_vscale_new_with_range (GAMMA_MIN, GAMMA_MAX, 0.01);
+    itf->rscale = rscale = gtk_vscale_new_with_range (GAMMA_MIN, GAMMA_MAX, 0.01);
     gtk_range_set_inverted (GTK_RANGE (rscale), TRUE);
     gtk_range_set_value (GTK_RANGE (rscale), redValue / 100.0);
     gtk_scale_set_digits (GTK_SCALE (rscale), 2);
@@ -654,7 +888,7 @@ run_dialog (McsPlugin * plugin)
     /* Green */
     glabel = gtk_label_new (_("Green"));
     gtk_table_attach (GTK_TABLE (table), glabel, 1, 2, 0, 1, GTK_FILL, GTK_FILL, BORDER, BORDER);
-    gscale = gtk_vscale_new_with_range (GAMMA_MIN, GAMMA_MAX, 0.01);
+    itf->gscale = gscale = gtk_vscale_new_with_range (GAMMA_MIN, GAMMA_MAX, 0.01);
     gtk_range_set_value (GTK_RANGE (gscale), greenValue / 100.0);
     gtk_range_set_inverted (GTK_RANGE (gscale), TRUE);
     gtk_scale_set_digits (GTK_SCALE (gscale), 2);
@@ -665,7 +899,7 @@ run_dialog (McsPlugin * plugin)
     /* Blue */
     blabel = gtk_label_new (_("Blue"));
     gtk_table_attach (GTK_TABLE (table), blabel, 2, 3, 0, 1, GTK_FILL, GTK_FILL, BORDER, BORDER);
-    bscale = gtk_vscale_new_with_range (GAMMA_MIN, GAMMA_MAX, 0.01);
+    itf->bscale = bscale = gtk_vscale_new_with_range (GAMMA_MIN, GAMMA_MAX, 0.01);
     gtk_range_set_value (GTK_RANGE (bscale), blueValue / 100.0);
     gtk_range_set_inverted (GTK_RANGE (bscale), TRUE);
     gtk_scale_set_digits (GTK_SCALE (bscale), 2);
@@ -674,17 +908,23 @@ run_dialog (McsPlugin * plugin)
     gtk_table_attach (GTK_TABLE (table), bscale, 2, 3, 1, 2, GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
 
     /* */
-    checkbox = gtk_check_button_new_with_label (_("Sync sliders"));
+    itf->synctoggle = checkbox = gtk_check_button_new_with_label (_("Sync sliders"));
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbox), syncGamma);
     gtk_table_attach (GTK_TABLE (table), checkbox, 0, 3, 2, 3, GTK_FILL, GTK_FILL, 0, BORDER);
 
 #ifdef USE_XF86VM
     if (haveXxf86vm)
     {
-        g_signal_connect (G_OBJECT (rscale), "value-changed", G_CALLBACK (redChangedCB), plugin);
-        g_signal_connect (G_OBJECT (gscale), "value-changed", G_CALLBACK (greenChangedCB), plugin);
-        g_signal_connect (G_OBJECT (bscale), "value-changed", G_CALLBACK (blueChangedCB), plugin);
-        g_signal_connect (G_OBJECT (checkbox), "toggled", G_CALLBACK (syncGammaChangedCB), plugin);
+        /* Store values to revert to */
+        itf->original_rgamma = redValue;
+        itf->original_ggamma = greenValue;
+        itf->original_bgamma = blueValue;
+        itf->original_sync = syncGamma;
+
+        g_signal_connect (G_OBJECT (rscale), "value-changed", G_CALLBACK (redChangedCB), itf);
+        g_signal_connect (G_OBJECT (gscale), "value-changed", G_CALLBACK (greenChangedCB), itf);
+        g_signal_connect (G_OBJECT (bscale), "value-changed", G_CALLBACK (blueChangedCB), itf);
+        g_signal_connect (G_OBJECT (checkbox), "toggled", G_CALLBACK (syncGammaChangedCB), itf);
     }
     else
 #endif
@@ -700,6 +940,9 @@ run_dialog (McsPlugin * plugin)
 
 
     /* */
+    gtk_widget_realize (dialog);
+    gdk_x11_window_set_user_time(GTK_WIDGET (dialog)->window, 
+            gdk_x11_get_server_time (GTK_WIDGET (dialog)->window));
     gtk_widget_show_all (dialog);
 }
 
